@@ -18,12 +18,13 @@
 
 import codecs
 import os
+import textwrap
 import sys, optparse
 from pyccuracy.core import PyccuracyCore
 from pyccuracy.common import Status
 from pyccuracy.story_runner import StoryRunner, ParallelStoryRunner
 from pyccuracy import Version, Release
-from pyccuracy.colored_terminal import ProgressBar
+from pyccuracy.colored_terminal import ProgressBar, TerminalController
 
 __version_string__ = "pyccuracy %s (release '%s')" % (Version, Release)
 __docformat__ = 'restructuredtext en'
@@ -34,12 +35,46 @@ sys.stdout = codecs.getwriter('utf-8')(sys.stdout)
 no_progress = False
 prg = None
 scenarios_ran = 0
+ctrl = TerminalController()
+
+def position(level, message, offset=10):
+    limit = 120 - (level * offset)
+    offset_message = (level * offset) * " "
+    separator = "\n%s" % offset_message
+    display_message = separator.join(textwrap.wrap(message, limit))
+    line = "%s%s" % (offset_message, display_message)
+    return line
+
+def before_action(context, action, args, kwarg):
+    pass
+
+def action_successful(context, action, args, kwarg):
+    print ctrl.render("${GREEN}%s" % position(1, action.description))
+
+def action_error(context, action, args, kwarg, error):
+    print ctrl.render("${RED}%s" % position(1, action.description))
+
+def scenario_started(fixture, scenario, scenario_index):
+    global scenarios_ran
+
+    total_scenarios = fixture.count_total_scenarios()
+    scenario_message = "Scenario %d of %d <%.2f%%> - %s" % (scenarios_ran + 1, total_scenarios, (float(scenarios_ran) / float(total_scenarios) * 100), scenario.title)
+    print
+    print ctrl.render("${NORMAL}%s" % position(0, scenario_message))
+
+def scenario_completed(fixture, scenario, scenario_index):
+    global scenarios_ran
+
+    scenarios_ran += 1
 
 def create_progress(verbosity):
     global no_progress
     global prg
     global scenarios_ran
-    
+
+    if verbosity == 3:
+        return
+
     scenarios_ran = 0
     if not no_progress:
         prg = ProgressBar("Pyccuracy - %s" % __version_string__, verbosity)
@@ -60,11 +95,14 @@ def update_progress(fixture, scenario, scenario_index):
             return
 
         current_progress = float(scenarios_ran) / total_scenarios
-        prg.update(current_progress, "Scenario %d of %d <%.2fs> - %s" % (scenarios_ran, total_scenarios, fixture.ellapsed(), scenario.title))
+        prg.update(current_progress, "[%s] Scenario %d of %d <%.2fs> - %s" % (scenario.status[0], scenarios_ran, total_scenarios, fixture.ellapsed(), scenario.title))
 
 def main(arguments=sys.argv[1:]):
     """ Main function - parses args and runs action """
     global no_progress
+    global scenarios_ran
+
+    scenarios_ran = 0
 
     extra_browser_driver_arguments = "\n\nThe following extra browser driver arguments " \
                                      " are supported in the key=value format:\n\nSelenium Browser Driver:\n" \
@@ -94,7 +132,9 @@ def main(arguments=sys.argv[1:]):
     parser.add_option("-R", "--report", dest="write_report", default="true", help="Should write report. Defines if Pyccuracy should write an html report after each run [default: %default].")
     parser.add_option("-D", "--reportdir", dest="report_dir", default=os.curdir, help="Report directory. Defines the directory to write the report in [default: %default].")
     parser.add_option("-F", "--reportfile", dest="report_file_name", default="report.html", help="Report file. Defines the file name to write the report with [default: %default].")
-    parser.add_option("-v", "--verbosity", dest="verbosity", default="2", help="Verbosity. 0 - does not show any output, 1 - shows text progress, 2 - shows animated progress bar")
+
+    #verbosity
+    parser.add_option("-v", "--verbosity", dest="verbosity", default="2", help="Verbosity. 0 - does not show any output, 1 - shows text progress, 2 - shows animated progress bar, 3 - Shows action by action")
 
     options, args = parser.parse_args(arguments)
 
@@ -112,7 +152,22 @@ def main(arguments=sys.argv[1:]):
             key, value = arg.split('=')
             extra_args[key] = value
 
-    create_progress(int(options.verbosity))
+    verbosity = int(options.verbosity)
+
+    create_progress(verbosity)
+
+    on_before_action_handler = None
+    on_action_successful_handler = None
+    on_action_error_handler = None
+    on_scenario_started_handler = None
+    on_scenario_completed_handler = update_progress
+
+    if verbosity == 3:
+        on_before_action_handler = before_action
+        on_action_successful_handler = action_successful
+        on_action_error_handler = action_error
+        on_scenario_started_handler = scenario_started
+        on_scenario_completed_handler = scenario_completed
 
     result = pyc.run_tests(actions_dir=options.actions_dir,
                            custom_actions_dir=options.custom_actions_dir,
@@ -131,8 +186,11 @@ def main(arguments=sys.argv[1:]):
                            should_throw=options.should_throw,
                            workers=workers,
                            extra_args=extra_args,
-                           on_scenario_started=None,
-                           on_scenario_completed=update_progress,
+                           on_scenario_started=on_scenario_started_handler,
+                           on_scenario_completed=on_scenario_completed_handler,
+                           on_before_action=on_before_action_handler,
+                           on_action_successful=on_action_successful_handler,
+                           on_action_error=on_action_error_handler,
                            verbosity=int(options.verbosity))
 
     if not result or result.get_status() != "SUCCESSFUL":
